/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "recv.cmdline.h"
#include "http.h"
#include "recv.h"

#define BUFSIZE 8192
enum {HTTP_CONNECTED, HTTP_SENT_GET_REQUEST, HTTP_STREAMING};

extern struct gengetopt_args_info conf;

static struct receiver *self; /* pointer to our own receiver struct */
static int status; /* one of the above */
static int fd; /* the socket fd */

/* no encryption */
void (*crypt_function_recv)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;
void (*crypt_function_send)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;

static void http_shutdown(void)
{
	free(self->buffer);
	close(fd);
}

static char *make_request_msg(void)
{
	char *ret, *hn = para_hostname();
	ret = make_message("%s\nHost: %s\nUser-Agent: para_recv/%s\n\n\n",
		HTTP_GET_MSG, hn, VERSION);
	free(hn);
	return ret;
}

static int http_setup_select_parms(fd_set *rfds, fd_set *wfds, __unused struct timeval *timeout)
{
	if  (status == HTTP_CONNECTED) {
		PARA_NOTICE_LOG("setting fd %d for write\n", fd);
		FD_SET(fd, wfds);
	} else
		FD_SET(fd, rfds);
	return fd;
}

static int http_handle_fds(int select_ret, fd_set *rfds, fd_set *wfds)
{
	int ret;

	if (!select_ret) /* we're not interested in timeouts */
		return 1;
	if  (status == HTTP_CONNECTED) {
		char *rq;
		if (!FD_ISSET(fd, wfds))
			return 1; /* nothing to do */
		rq = make_request_msg();
		PARA_NOTICE_LOG("sending request message:\n%s\n", rq);
		ret = send_va_buffer(fd, "%s", rq);
		free(rq);
		if (ret < 0)
			return E_WRITE;
		status = HTTP_SENT_GET_REQUEST;
		return 1;
	}
	if (!FD_ISSET(fd, rfds))
		return 1; /* nothing to do */
	if (status == HTTP_SENT_GET_REQUEST) {
		ret = recv_pattern(fd, HTTP_OK_MSG, MAXLINE);
		if (ret < 0)
			return -E_MISSING_OK;
		PARA_NOTICE_LOG("%s", "received ok msg, streaming\n");
		status = HTTP_STREAMING;
		return 1;
	}
	/* already streaming */
	if (!self->buffer)
		self->buffer = para_malloc(BUFSIZE);
	if (self->loaded >= BUFSIZE)
		return -E_OVERRUN;
	ret = recv_buffer(fd, self->buffer, BUFSIZE - self->loaded);
	if (ret <= 0)
		return ret < 0? -E_READ : 0;
	self->loaded += ret;
	return 1;
}

int http_recv_init(struct receiver *r)
{
	struct hostent *he;
	struct sockaddr_in their_addr;

	/* get the host info */
	PARA_NOTICE_LOG("getting host info of %s\n", conf.http_host_arg);
	if (!(he = get_host_info(conf.http_host_arg)))
		return -E_HOST_INFO;
	/* get new socket */
	if ((fd = get_socket()) < 0)
		return -E_SOCKET;
	/* init their_addr */
	init_sockaddr(&their_addr, conf.http_port_arg, he);
	/* connect */
	PARA_NOTICE_LOG("connecting to %s...\n", conf.http_host_arg);
	if (do_connect(fd, &their_addr) < 0) {
		PARA_ERROR_LOG("connect failed (%s)\n", strerror(errno));
		return -E_CONNECT;
	}
	status = HTTP_CONNECTED;
	r->setup_select_parms = http_setup_select_parms;
	r->handle_fds = http_handle_fds;
	r->shutdown = http_shutdown;
	r->buffer = NULL;
	r->loaded = 0;
	self = r;
	return 1;
}
